==> lkrg-0.5.tar.gz: lkrg-0.5/src/modules/exploit_detection/syscalls/p_sys_setfsuid/p_sys_setfsuid.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept setfsuid syscall
 *
 * Notes:
 *  - Exploit Detection features needs to whitelist process which
 *    legitimately increases their privileges (e.g. 'su', 'sudo', etc.)
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 09.IX.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_sys_setfsuid_kretprobe_state = 0x0;

static struct kretprobe p_sys_setfsuid_kretprobe = {
    .kp.symbol_name = P_GET_SET_ID_NAME(setfsuid),
    .handler = p_sys_setfsuid_ret,
    .entry_handler = p_sys_setfsuid_entry,
    .data_size = sizeof(struct p_sys_setfsuid_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_setfsuid_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_sys_setfsuid_entry>\n");

   /*
    * This is special case, since we can NOT identify when syscall fail or succeed,
    * we must verify process list before and after the syscall
    */
   p_ed_enforce_validation();
   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_debug_kprobe_log(
          "Leaving function <p_sys_setfsuid_entry>\n");

   return 0x0;
}


int p_sys_setfsuid_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_sys_setfsuid_ret>\n");
   p_debug_kprobe_log(
          "Setfsuid returned value => %ld comm[%s] Pid:%d parent[%d]\n",
                       p_regs->ax,current->comm,current->pid,current->real_parent->pid);

   // Update process
//   if (!p_regs->ax) {
   spin_lock(&p_rb_ed_pids_lock);
      if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
         // This process is on the ED list - update information!
         p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",current->pid);

         p_update_ed_process(p_tmp, current);
         p_set_ed_process_on(p_tmp);
      }
//   }
   spin_unlock(&p_rb_ed_pids_lock);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_sys_setfsuid_ret>\n");

   return 0x0;
}


int p_install_sys_setfsuid_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_sys_setfsuid_hook>\n");

   if ( (p_ret = register_kretprobe(&p_sys_setfsuid_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_sys_setfsuid_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_sys_setfsuid_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_sys_setfsuid_kretprobe.kp.symbol_name,
               p_sys_setfsuid_kretprobe.kp.addr);
   p_sys_setfsuid_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_sys_setfsuid_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_sys_setfsuid_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_sys_setfsuid_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_sys_setfsuid_hook>\n");

   if (!p_sys_setfsuid_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_sys_setfsuid_kretprobe.kp.symbol_name,
                  p_sys_setfsuid_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_sys_setfsuid_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_sys_setfsuid_kretprobe.kp.symbol_name,
                  p_sys_setfsuid_kretprobe.kp.addr,
                  p_sys_setfsuid_kretprobe.nmissed);
      p_sys_setfsuid_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_sys_setfsuid_hook>\n");
}

==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/exploit_detection/syscalls/p_sys_setfsuid/p_sys_setfsuid.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept setfsuid syscall
 *
 * Notes:
 *  - None
 *
 * Caveats:
 *  - None
 *
 * Timeline:
 *  - Created: 09.IX.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../p_lkrg_main.h"


char p_sys_setfsuid_kretprobe_state = 0x0;

static struct kretprobe p_sys_setfsuid_kretprobe = {
    .kp.symbol_name = P_GET_SET_ID_NAME(setfsuid),
    .handler = p_sys_setfsuid_ret,
    .entry_handler = p_sys_setfsuid_entry,
    .data_size = sizeof(struct p_sys_setfsuid_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_sys_setfsuid_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_setfsuid_entry>\n");

   /*
    * This is special case, since we can NOT identify when syscall fail or succeed,
    * we must verify process list before and after the syscall
    */
   p_ed_enforce_validation();

   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

   p_debug_kprobe_log(
          "Leaving function <p_sys_setfsuid_entry>\n");

   return 0x0;
}


int p_sys_setfsuid_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_sys_setfsuid_ret>\n");
   p_debug_kprobe_log(
          "Setfsuid returned value => %ld comm[%s] Pid:%d parent[%d]\n",
                       p_regs_get_ret(p_regs),current->comm,current->pid,current->real_parent->pid);

   // Update process
//   if (!p_regs_get_ret(p_regs)) {
   p_tasks_write_lock(&p_flags);
      if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
         // This process is on the ED list - update information!
         p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",current->pid);

         p_update_ed_process(p_tmp, current, 0);
         p_set_ed_process_on(p_tmp);
      }
//   }
   p_tasks_write_unlock(&p_flags);

//   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_sys_setfsuid_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(sys_setfsuid)
